unit erfassenUnit;

interface

uses
  System.SysUtils, System.Classes, JS, Web, WEBLib.Graphics, WEBLib.Controls,
  WEBLib.Forms, WEBLib.Dialogs, Vcl.StdCtrls, WEBLib.StdCtrls, Vcl.Controls,
  WEBLib.ExtCtrls, Data.DB, WEBLib.DB, WEBLib.CDS, WEBLib.REST, WEBLIB.JSON,
  WEBLib.Devices, System.NetEncoding, WEBLib.LocalFiles, WEBLib.ClientConnector,
  WEBLib.FirestoreCDS, WEBLib.IndexedDb, WEBLib.Grids, WEBLib.DBCtrls, Vcl.Grids,
  Vcl.Imaging.jpeg, WEBLib.Actions;

type

    TForm1 = class(TWebForm)
        BarcodeDecoder: TWebBarcodeDecoder;
        WebCamera: TWebCamera;
        WebElementActionList1: TWebElementActionList;
    wcbWebCam: TWebComboBox;
        WebLabel1: TWebLabel;
        weAbNr: TWebEdit;
        wbScanAb: TWebButton;
        wbScanSn: TWebButton;

        procedure WebFormCreate(Sender: TObject);
        procedure wbScanAbClick(Sender: TObject);
        procedure wbScanSnClick(Sender: TObject);
        procedure BarcodeDecoderDecoded(Sender: TObject; ADecoded: string);
        procedure WebCameraCameraDevicesInitialized(Sender: TObject);
        procedure WebCameraSwitchCamera(Sender: TObject; ACamera: TCameraDevice);
        procedure wcbWebCamChange(Sender: TObject);
        procedure WebFormShow(Sender: TObject);
        procedure saveImgInDB(data : string; url : string);
        procedure WebElementActionList1Actions1Execute(Sender: TObject; Element: TJSHTMLElementRecord; Event: TJSEventParameter);

        function testRegEx(input : string) : boolean;

    private
        FJSONObject: TJSONObject;
        fJSONArray : TJSONArray;
    public
        Auftrag                 : string;
  end;

var
    IndexedDbClientDataset  : TWebIndexedDbClientDataset;
    DataSource              : TWebDataSource;

    Form1                   : TForm1;

implementation

{$R *.dfm}

uses
    System.IOUtils,
    bearbeitenUnit;


{ ***************************************************************************
 *                Format Auftragsnummer berprfen                           *
  ***************************************************************************  }
function  TForm1.testRegEx(input : string) : boolean;
var
    testErgebnis : boolean;
    temp : string;
begin
    //temp := input;
    asm
        var testErgebnis;
        const myReg = /^AB{1}-{1}[0-9]{5}-{1}2{1}[2-9]{1}\/A{1}[0-9]{1,2}$/;    // AB-xxxxx-2x/Axx
        testErgebnis = myReg.test(input);                                       // AB-00000-23/A00  von                                                                                // AB-99999-29/A99  bis
    end;                                                                        // AB-99999-29/A99  bis
    result := testErgebnis;
end;

{ ***************************************************************************
 *   Wenn die Form erzeugt wird                                              *
  ***************************************************************************  }
procedure TForm1.WebFormCreate(Sender: TObject);
begin
     // Setup permanent indexes
    IndexedDBClientDataSet.AddIDBIndex('AB_Nr', 'ab');
    IndexedDBClientDataSet.AddIDBIndex('Anlage', 'anlage');
    IndexedDBClientDataSet.AddIDBIndex('Datum', 'date');
    IndexedDBClientDataSet.AddIDBIndex('Zeit', 'time');
    IndexedDBClientDataSet.AddIDBIndex('ImageData', 'imageData');
    IndexedDBClientDataSet.AddIDBIndex('ImageUrl', 'imageUrl');
    IndexedDBClientDataSet.AddIDBIndex('ImageLen', 'imageLen');

    IndexedDBClientDataSet.IDBActiveIndex := 'AB_Nr';

    DataSource.DataSet :=  IndexedDBClientDataSet;

    IndexedDBClientDataSet.FieldDefs.Clear;

    // add key field
    IndexedDBClientDataSet.FieldDefs.Add('id',ftInteger, 0, true);
    // make it a hidden column
    IndexedDBClientDataSet.FieldDefs.Items[0].Attributes :=  IndexedDBClientDataSet.FieldDefs.Items[0].Attributes + [faHiddenCol];

    // add normal fields
    IndexedDBClientDataSet.FieldDefs.Add('ab',ftString);
    IndexedDBClientDataSet.FieldDefs.Add('anlage',ftstring);
    IndexedDBClientDataSet.FieldDefs.Add('date',ftDate);
    IndexedDBClientDataSet.FieldDefs.Add('time',ftTime);
    IndexedDBClientDataSet.FieldDefs.Add('imageData',ftString);
    IndexedDBClientDataSet.FieldDefs.Add('imageUrl',ftString);
    IndexedDBClientDataSet.FieldDefs.Add('imageLen',ftInteger);

    weAbNr.Text     := '';

    Application.ErrorType :=aeSilent;

    WebCamera.Start;

    BarcodeDecoder.Reader := [bcvCode128];

end;

{ ***************************************************************************
 *                   Erfasse AB-Nummer                                       *
  ***************************************************************************  }
procedure TForm1.wbScanAbClick(Sender: TObject);
var
    ImageData       : string;
    ImageDataUrl    : string;
begin
    BarcodeDecoder.DecodeURL(WebCamera.SnapShotAsBase64);
    ImageData       := WebCamera.SnapShotAsBase64;
    ImageDataUrl    := 'data:image/jpeg;base64,';
end;

{ ***************************************************************************
 *                   Foto in Indexdb ablegen                                 *
  ***************************************************************************  }
procedure TForm1.saveImgInDB(data : string; url : string);
var
    ImageData       : string;
    ImageDataUrl    : string;
    ImageDataLen    : string;

    Filename        : string;
    Anlage          : string;
    Auftragsnummer  : string;

    Posx            : integer;
    len             : integer;

begin
    // Actuelles Bild in Vorschau
    // WebImageControl.Url := WebCamera.SnapShotAsBase64;
    try
        ImageData       := data;
        ImageDataUrl    := url;

        len := length(ImageData);

        Posx := Pos('/', weAbNr.Text);
        Auftragsnummer  := weAbNr.Text;
        Auftragsnummer  := Auftragsnummer.Substring(0,Posx - 1);

        Anlage  := weAbNr.Text;
        Anlage  := Anlage.Substring(Posx, length(Anlage) - Posx);

        // make filename
        FileName := 'C:/temp/'      +                                               // Pfad
                     weAbNr.Text + '_' +                                            // Auftrag
                     FormatDateTime('dd-mm-yyyy', now) + '_' +                      // Datum
                     FormatDateTime('hh-nn-ss', now) +                              // Zeit
                     '.jpg' ;                                                       // Datei Suffix

        // neuen Datensatz anlegen
        IndexedDBClientDataSet.Insert;
        IndexedDBClientDataSet.FieldByName('ab').AsString := Auftragsnummer;
        IndexedDBClientDataSet.FieldByName('anlage').AsString := Anlage;
        IndexedDBClientDataSet.FieldByName('date').AsDateTime := now;
        IndexedDBClientDataSet.FieldByName('time').AsDateTime := now;
        IndexedDBClientDataSet.FieldByName('imageData').AsString := ImageData;
        IndexedDBClientDataSet.FieldByName('imageUrl').AsString := ImageDataUrl;
        IndexedDBClientDataSet.FieldByName('imageLen').AsString := IntToStr(len);

        IndexedDBClientDataSet.Post;
    except
        ShowMessage('Error beim speichern des Bildes in der IndexedDB !');
    end;
end;


{ ***************************************************************************
 *                    Foto erzeugen und sichern                              *
  ***************************************************************************  }
procedure TForm1.wbScanSnClick(Sender: TObject);
var
    ImageData       : string;
    ImageDataUrl    : string;
begin
     if testRegEx(weAbNr.Text) then // AB-Nummer berprfen
     begin
        Auftrag         := weAbNr.Text;
        ImageData       := WebCamera.SnapShotAsBase64;
        ImageDataUrl    := 'data:image/jpeg;base64,';
        saveImgInDB(ImageData, ImageDataUrl);
        ShowMessage('Bild Erzeugt und abgelegt !');
     end
        else
     begin
        // warn Hinweis AB-Nummer ????
        ShowMessage('Eingabeformat stimmt nicht, AB-Nummer soll -> AB-xxxxx-2x/Axx');
        asm
            var obj = null;
            obj = document.getElementById('AB_Nummer').focus();
        end;
     end;
 end;

{ ***************************************************************************
 *                    Foto von Code erzeugen und sichern                     *
  ***************************************************************************  }
procedure TForm1.BarcodeDecoderDecoded(Sender: TObject; ADecoded: string);
var
    ImageData       : string;
    ImageDataUrl    : string;
begin
    try
        weAbNr.Text     := ADecoded;
        Auftrag         := weAbNr.Text;
        ImageData       := WebCamera.SnapShotAsBase64;
        ImageDataUrl    := 'data:image/jpeg;base64,';
        saveImgInDB(ImageData, ImageDataUrl);
    except
        ShowMessage('Error beim Code decodieren !');
    end;
end;

{ ***************************************************************************
 *            Wenn die Cam initialisiert ist combobox fllen                 *
  ***************************************************************************  }
procedure TForm1.WebCameraCameraDevicesInitialized(Sender: TObject);
var
    I: Integer;
    d: TCameraDevice;
begin
    try
        for I := 0 to WebCamera.CameraDevices.Count - 1 do
            wcbWebCam.Items.Add(WebCamera.CameraDevices.Items[I].Name);
    except
        ShowMessage('Error beim Cam initialisieren !');
    end;
end;

{ ***************************************************************************
 *                    WebCam starten                                         *
  ***************************************************************************  }
procedure TForm1.WebCameraSwitchCamera(Sender: TObject; ACamera: TCameraDevice);
begin
    try
        WebCamera.Start;
    except
        ShowMessage('Error beim Cam starten !');
    end;
end;

{ ***************************************************************************
 *                    Wenn WebCam gewechselt werden soll                     *
  ***************************************************************************  }
procedure TForm1.wcbWebCamChange(Sender: TObject);
var
    d: TCameraDevice;
begin
    try
        d := WebCamera.CameraDevices.GetDeviceByName(wcbWebCam.Items[wcbWebCam.ItemIndex]);
        WebCamera.SetSelectedCameraDevice(d);
    except
        ShowMessage('Error beim Cam umschalten !');
    end;
end;

{ ***************************************************************************
 *                    Umschalten auf bearbeiten Form                         *
  ***************************************************************************  }
procedure TForm1.WebElementActionList1Actions1Execute(Sender: TObject;
  Element: TJSHTMLElementRecord; Event: TJSEventParameter);
begin
    {ID=siee2}
    IndexedDBClientDataSet.Active := false;
    Application.CreateForm(TForm2, Form2);
end;

{ ***************************************************************************
 *                    Wenn die Form angezeigt wird                           *
  ***************************************************************************  }
procedure TForm1.WebFormShow(Sender: TObject);
var
    el: TJSElement;
begin
  //application.OnImageCacheReady := nil;
  IndexedDBClientDataSet.Active := true;

  el := document.getElementById('side1');       // todo
  el['class'] := 'active';                      // todo

  weAbNr.Text := Auftrag;
end;

end.
